package controller;

import java.awt.Dimension;
import java.awt.Graphics;
import java.io.*;

import model.BambooModel;
import model.SaveController;

import ui.*;

public class ApplicationController extends BambooController {
	private SaveController saveController;
	private ApplicationView view;

	public ApplicationController(BambooModel model) {
		super(model);
		saveController = model.getSaveController();
		view = new ApplicationView(this, model);
		setView(view);
		view.createView();
	}

	@Override
	public void newFrame(int masu, int size) {
		super.newFrame(masu, size);
		view = new ApplicationView(this, model);
		view.setParameter(masu, masu, size);
		setView(view);
		view.createView();
	}

	public void write(Graphics g) {
		int left = saveController.getSaveX();
		int right = left + saveController.getSaveWidth();
		int top = saveController.getSaveY();
		int bottom = top + saveController.getSaveHeight();
		view.write(g, left, right, top, bottom);
	}

	protected File newFile(File file, String[] strings) {
		String fileName = file.getPath(); // tpX̃t@C𓾂
		String lowerName = fileName.toLowerCase();

		for (int i = 0; i < strings.length; i++) {
			if (lowerName.endsWith(strings[i])) {
				return file;
			}
		}

		fileName += "." + strings[0];

		return new File(fileName);
	}

	public void saveAtf(File file) {
		File newFile = newFile(file, new String[] { "atf" });
		if (view.canReplaceFile(newFile)) {
			saveController.saveAtf(newFile); // Z[ũ\bhĂяo
		}
	}

	public Dimension getSaveSquare() {
		/*
		 * saveController.setSquare(); int width = saveController.getSaveWidth() *
		 * BambooView.sizeOfOriginal + 1; // 摜̕ int height =
		 * saveController.getSaveHeight() * BambooView.sizeOfOriginal + 1;
		 */
		int[] rectangle = saveController.getRectangle();
		// return new Dimension(width, height);
		return new Dimension(rectangle[0], rectangle[1]);
	}

	public Dimension getSaveRegion() {
		saveController.setSaveXY();
		int width = saveController.getSaveWidth() * BambooView.sizeOfOriginal
				+ 1;
		int height = saveController.getSaveHeight() * BambooView.sizeOfOriginal
				+ 1;
		return new Dimension(width, height);
	}

	public void url() {
		String url = saveController.getURL();
		view.showURL(url);
	}

	public void saveURL(String text) {
		saveController.saveURL(text);
	}

	public void save() {
		String fileName = ((ApplicationView) view).save();
		if (fileName == null) {
			return;
		}
		String lowerName = fileName.toLowerCase(); // ꍇ̂߂ɏp
		File file = new File(fileName);

		/* gqɂăt@C */
		if (lowerName.endsWith(".png")) {
			saveImageEncode(file, "png");
			return;
		}

		if (lowerName.endsWith(".bmp")) {
			saveImageEncode(file, "bmp");
			return;
		}

		// "jpeg"Ȃ瓧
		if (lowerName.endsWith(".jpeg")) {
			saveImageEncode(file, "jpeg");
			return;
		}

		// "jpg"Ȃ琶
		if (lowerName.endsWith(".jpg")) {
			saveJpg(file);
			return;
		}

		if (lowerName.endsWith(".atf")) {
			saveAtf(file);
			return;
		}

		if (lowerName.endsWith(".dat")) {
			// layerList.saveData(file);
			model.getLayerList().saveData(file);
			return;
		}
	}

	private void saveJpg(File jpgFile) {
		if (view.canReplaceFile(jpgFile)) {
			view.saveImage(jpgFile, "jpeg");
		}
	}

	private void saveImageEncode(File file, String suffix) {
		if (view.canReplaceFile(file)) {
			encode(file, suffix);
		}
	}

	private void encode(File file, String format) {
		// layerList.encode(file, format, saveController, view);
		model.getLayerList().encode(file, format, saveController, view);
	}

}
