package model;

import java.io.*;
import java.util.*;

import javax.swing.table.AbstractTableModel;

import ui.*;
import utils.CellUtils;
import zuozuomu.watermark.*;

@SuppressWarnings("serial")
public class Board extends AbstractTableModel {
	private Cell[][] cells;

	private BambooParser parser = new BambooParser();

	public Board(final int width, final int height) {
		initialize(width, height);
	}

	public Board(final Cell[][] map) {
		initialize(map[0].length, map.length);
		for (int i = 0; i < map.length; i++)
			for (int j = 0; j < map[i].length; j++)
				cells[i + 1][j + 1].setNumber(map[i][j].getNumber());
	}

	protected void initialize(final int width, final int height) {
		cells = new Cell[height + 2][width + 2];
		for (int i = 0; i < cells[0].length; i++) {
			cells[0][i] = new Cell(Cell.KABE);
			cells[cells.length - 1][i] = new Cell(Cell.KABE);
		}
		for (int i = 1; i < cells.length - 1; i++) {
			cells[i][0] = new Cell(Cell.KABE);
			cells[i][cells[i].length - 1] = new Cell(Cell.KABE);
		}
		for (int i = 1; i < cells.length - 1; i++) {
			for (int j = 1; j < cells[i].length - 1; j++) {
				cells[i][j] = new Cell(Cell.KARA);
			}
		}
	}

	public String getData() {
		String data = "";
		for (int j = 1; j <= cells.length; j++) {
			for (int i = 1; i <= cells[0].length; i++) {
				data += cells[j][i] + " ";
			}
		}
		return data;
	}

	/** * very old ** */
	public void loadDat(File file) {
		try {
			FileReader fileReader = new FileReader(file);
			BufferedReader reader = new BufferedReader(fileReader);

			String line = null;

			while ((line = reader.readLine()) != null) {
				int i = 1;
				int j = 1;
				StringTokenizer parser = new StringTokenizer(line, " ");
				while (parser.hasMoreTokens()) {
					cells[j][i].setNumber(Integer.parseInt(parser.nextToken()));

					i++;
					if (i > cells[j].length) {
						i = 1;
						j++;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/** * dangerous ** */
	public int length() {
		return cells.length;
	}

	public void deleteAllIslands() {
		for (int j = 1; j < cells.length - 1; j++) {
			for (int i = 1; i < cells[0].length - 1; i++) {
				// if (isIsland(i, j)) {
				if (CellUtils.isIsland(cells[j][i])) {
					cells[j][i].setNumber(Cell.MASU);
				}
			}
		}
	}

	public void setMap(final int number, int xMin, int xMax, int yMin, int yMax) {
		for (int j = yMin; j <= yMax; j++) {
			for (int i = xMin; i <= xMax; i++) {
				cells[j][i].setNumber(number);
			}
		}
	}

	public void setMap(final int[][] map, int xstart, int xend, int ystart,
			int yend) {
		for (int i = 0; i < yend; i++) {
			for (int j = 0; j < xend; j++) {
				cells[i + ystart][j + xstart].setNumber(map[i][j]);
			}
		}

	}

	@Override
	public String toString() {
		return "Model" + (cells.length - 2) + "x" + (cells[0].length - 2);
	}

	/** image */
	public void encode(File file, String format, SaveController saveController,
			ApplicationView view) {
		JikkenImage image = new JikkenImage(view.createSaveImage());
		image.setSegmentation();
		byte[] header = { (byte) saveController.getSaveWidth(),
				(byte) saveController.getSaveHeight() };
		image.embedIntArray(0, 0, header); // (0,0)wb_f[^𖄂ߍ
		image.embedIntSquareArray(0, 1, cells, saveController.getSaveY(),
				saveController.getSaveX(), saveController.getSaveHeight(),
				saveController.getSaveWidth()); // (0,1)Ֆʃf[^ߍ
		image.writeJpegFile(file, format);
	}

	public void saveAtf(PrintWriter writer, int x, int y, int saveWidth,
			int saveHeight) {
		for (int j = 0; j < saveHeight; j++) {
			for (int i = 0; i < saveWidth; i++) {
				// if (cells[j + y][i + x] < 0) {
				// BigInteger ii;
				// ii.co
				// ii.compareTo(val);
				if (cells[j + y][i + x].compareTo(0) < 0) {
					writer.println("(" + (j + 1) + "," + (i + 1) + "):"
					// + (-cells[j + y][i + x]));
							+ (-cells[j + y][i + x].getNumber()));
				}
			}
		}
	}

	public int getMap(int i, int j) {
		return cells[j][i].getNumber();
	}

	public String getURL(int x, int y, int saveWidth, int saveHeight) {
		return parser.parse1(cells, x, y, saveWidth, saveHeight);
	}

	public void loadUrl(String query) {
		cells = parser.parseMap(query);
	}

	public Cell[][] getMap() {
		return cells;
	}

	public void reverse() {
		for (int row = 1; row < cells.length - 1; row++) {
			for (int column = 1; column <= (cells[row].length - 1) / 2; column++) {
				CellUtils.swap(cells[row][column], cells[row][cells[row].length
						- 1 - column]);
			}
		}
	}

	public void rotate() {
		Cell[][] map_new = new Cell[cells[0].length][cells.length];
		for (int i = 0; i < cells.length; i++) {
			for (int j = 0; j < cells[i].length; j++) {
				map_new[j][cells.length - 1 - i] = new Cell(CellUtils
						.changeRotate(cells[i][j].getNumber()));
			}
		}
		cells = map_new;
	}

	public int[][] toArray() {
		int[][] array = new int[cells.length - 2][cells[0].length - 2];
		for (int i = 0; i < array.length; i++) {
			System.arraycopy(cells[i + 1], 1, array[i], 0, array[i].length);
		}
		return array;
	}

	public int getColumnCount() {
		return cells[0].length - 2;
	}

	public int getRowCount() {
		return cells.length - 2;
	}

	public Object getValueAt(int row, int column) {
		return cells[row + 1][column + 1];
	}

	public Cell getCell(final int i, final int j) {
		return cells[j][i];
	}

	public void setValue(int value, int i, int j) {
		cells[j][i].setNumber(value);
	}

	/** bN[h̓ */
	/*
	public void setValueLocked(int value, int i, int j) {
		if (!cells[j][i].isNumber())
			cells[j][i].setNumber(value);
	}
*/
}
